package com.connormahaffey.TerraCraft;

/**
 *
 * @author Connor
 */
public class Block{
    public static final int SIZE = 20;
    public static final int AIR = 0, STONE = 1, DIRT = 2, GRASS = 3, WATER = 4, SAND = 5, BEDROCK = 6, WOOD = 7,
                            LEAVES = 8, CLOUD = 9, GOLD = 10;
    private final double[] HARDNESS_VALUES = {-1, 13, 4, 2, -1, 2, -1, 7, 2, -1, 25};

    public static final int FOREGROUND = 0, BACKGROUND = 1;
    private int type, layer, oldBackgroundType;
    private boolean playerPlaced;
    private double hardness;
    private Location location;

    public Block(int type, int layer, boolean playerPlaced, Location location){
        this.type = type;
        this.layer = layer;
        hardness = HARDNESS_VALUES[type];
        this.playerPlaced = playerPlaced;
        this.location = location;
        oldBackgroundType = AIR;
    }

    public Block(int type, int layer, boolean playerPlaced, Location location, int oldBackgroundType){
        this.type = type;
        this.layer = layer;
        hardness = HARDNESS_VALUES[type];
        this.playerPlaced = playerPlaced;
        this.location = location;
        this.oldBackgroundType = oldBackgroundType;
    }

    public Block(double[] data){
        location = new Location(data[0], data[1]);
        type = (int)data[2];
        layer = (int)data[3];
        hardness = HARDNESS_VALUES[type];
        if(data[4] == 1){
            playerPlaced = true;
        }
        else{
            playerPlaced = false;
        }
        oldBackgroundType = (int)data[5];
    }

    public int getType(){
        return type;
    }

    public int getLayer(){
        return layer;
    }

    public double getHardness(){
        return hardness;
    }

    public Location getLocation(){
        return location;
    }

    public boolean isSolid(){
        if(type == AIR || type == WATER || layer == BACKGROUND){
            return false;
        }
        else{
            return true;
        }
    }

    public boolean isPlacedByPlayer(){
        return playerPlaced;
    }

    public void setOldBackgroundType(int oldBackgroundType){
        this.oldBackgroundType = oldBackgroundType;
    }

    public int getOldBackgroundType(){
        return oldBackgroundType;
    }

    private int getPlayerPlaced(){
        if(playerPlaced){
            return 1;
        }
        else{
            return 0;
        }
    }

    @Override
    public String toString(){
        return location.getX() + "," + location.getY() + "," + type + "," + layer + "," + getPlayerPlaced() + "," + oldBackgroundType;
    }
}
