package com.connormahaffey.TerraCraft;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferStrategy;

/**
 *
 * @author Connor
 */
public class Game extends Canvas{

    public final static int INV_SCREEN = 0, LOADING_SCREEN = 1, GAME_SCREEN = 2;
    private int currentScreen;
    private boolean running, pause;
    private BufferStrategy bufferStrategy;
    
    private World world;
    private WorldLogic worldLogic;
    private GameScreen gameScreen;
    private PlayerLogic playerLogic;
    private MouseLogic mouseLogic;
    private OverlayLogic overlayLogic;
    private InvScreen overlay;
    private GameOverlayScreen gameOverlay;

    public Game(){
        running = false;
        pause = false;
    }

    public void init(){
        bufferStrategy = this.getBufferStrategy();
        currentScreen = 1;
        world = new World();
        gameOverlay = new GameOverlayScreen(world);
        gameOverlay.init();
        worldLogic = new WorldLogic(this, world);
        Thread worldLogicThread = new Thread(worldLogic);
        worldLogicThread.start();

    }

    public void initGameScreen(){
        gameScreen = new GameScreen(world);
        gameScreen.init();
        playerLogic = new PlayerLogic(world);
        overlay = new InvScreen(playerLogic);
        overlay.init();
        mouseLogic = new MouseLogic(world);
        overlayLogic = new OverlayLogic(this, playerLogic);

        Thread playerLogicThread = new Thread(playerLogic);
        playerLogicThread.start();

        Thread mouseLogicThread = new Thread(mouseLogic);
        mouseLogicThread.start();

        Thread overlayLogicThread = new Thread(overlayLogic);
        overlayLogicThread.start();
        //these are initialized here because otherwise they get erased
        FileHandler fileHandler = new FileHandler();
        TerraCraft.player.setCreative(fileHandler.getBool(world.getData(), "CreativeMode"));
        MouseLogic.setInstaBreak(fileHandler.getBool(world.getData(), "InstantBreak"));
        if(fileHandler.getBool(world.getData(), "SuperJump")){
            PlayerLogic.setSpeed(.60);
        }
        else{
            PlayerLogic.setSpeed(.30);
        }
        
        currentScreen = GAME_SCREEN;
        TerraCraft.keyHandler.setMode(KeyHandler.MODE_GAME);
    }

    public void loop(){
        Graphics2D graphics;
        running = true;
        int fps = 0, frames = 0;
        long time = System.currentTimeMillis();
        boolean erase = true;

        while(running){
            graphics = (Graphics2D)bufferStrategy.getDrawGraphics();
            graphics.clearRect(0, 0, TerraCraft.SIZE_X, TerraCraft.SIZE_Y);

            while(pause){
                try{
                    Thread.sleep(50);
                }catch(Exception e){
                    e.printStackTrace();
                }
            }

            if(currentScreen == GAME_SCREEN){
                gameScreen.draw(graphics);
                gameOverlay.draw(graphics);
            }
            else if(currentScreen == INV_SCREEN){
                gameScreen.draw(graphics); //draw game underneath menu
                overlay.draw(graphics); //draw inventory overlay
            }
            else{
                graphics.drawString("Loading...", TerraCraft.HALF_X - 20, TerraCraft.HALF_Y);
            }

            frames++;

            if(System.currentTimeMillis() - time >= 1000){
                fps = frames;
                frames = 0;
                time = System.currentTimeMillis();
            }

            graphics.setColor(Color.BLACK);
            graphics.drawString("FPS: " + fps, TerraCraft.SIZE_X - 80, TerraCraft.SIZE_Y - 40);

            bufferStrategy.show();
            graphics.dispose();
        }
    }

    public void pause(){
        pause = true;
        playerLogic.pause();
        mouseLogic.pause();
        overlayLogic.pause();
    }

    public void resume(){
        pause = false;
        playerLogic.resume();
        mouseLogic.resume();
        overlayLogic.resume();
    }

    public void exit(){
        running = false;//should be done last
        worldLogic.stop();
        playerLogic.stop();
        mouseLogic.stop();
        overlayLogic.stop();
        System.exit(0);
    }

    public int getScreen(){
        return currentScreen;
    }

    public void setScreen(int numScreen){
        currentScreen = numScreen;
    }

}
