package com.connormahaffey.TerraCraft;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.util.ArrayList;

/**
 *
 * @author Connor
 */
public class GameScreen {

    private World world;
    private ArrayList<Chunk> chunks;
    private Chunk chunk;
    private ArrayList<Block> blocks, blocksDrawAfter;
    private ArrayList<int[]> blocksDrawAfterSpot;
    private Block block;
    private Location loc;

    private Color skyColor, darkness;

    private BasicStroke circleStroke;

    private int ySpot, xSpot, x, y, temp;
    private int[] tempList;

    public GameScreen(World world){
        this.world = world;
    }

    public void init(){
        skyColor = new Color(0, 164, 219);
        darkness = new Color(0, 0, 0, 95);
        circleStroke = new BasicStroke(2f);
        blocksDrawAfter = new ArrayList<Block>();
        blocksDrawAfterSpot = new ArrayList<int[]>();
    }

    public void draw(Graphics2D g){
        x = (int)Math.floor(TerraCraft.player.getX());
        y = (int)Math.floor(TerraCraft.player.getY());
        g.setColor(skyColor);
        g.fillRect(0, 0, TerraCraft.SIZE_X, TerraCraft.SIZE_Y);
        //avoid foreach loops for performance reasons
        chunks = world.getChunks();
        for(int i = 0; i < chunks.size(); i++){
            chunk = chunks.get(i);
            blocks = chunk.getBlocks();
            for(int j = 0; j < blocks.size(); j++){
                block = blocks.get(j);
                loc = block.getLocation();
                ySpot = loc.getYInt() + TerraCraft.OFF_Y - y;
                xSpot = loc.getXInt() + TerraCraft.OFF_X - x;
                if(ySpot <= TerraCraft.SIZE_Y &&
                   ySpot >= -World.BLOCK_SIZE &&
                   xSpot <= TerraCraft.SIZE_X &&
                   xSpot >= -World.BLOCK_SIZE){
                    if(block.getType() != Block.AIR){
                        if(block.getType() == Block.WATER){
                            tempList = new int[2];
                            tempList[0] = xSpot;
                            tempList[1] = ySpot;
                            blocksDrawAfterSpot.add(tempList);
                            blocksDrawAfter.add(block);
                        }
                        else{
                            g.drawImage(world.getBlockImage(block),
                                        xSpot,
                                        ySpot,
                                        null);
                        }
                    }
                    //darkness - kills fps - drops me from 280-300 to 70-110
                    //g.setColor(darkness);
                    //g.fillRect(xSpot, ySpot, World.BLOCK_SIZE, World.BLOCK_SIZE);
                }
            }
        }
        //draw player
        g.drawImage(TerraCraft.player.getImage(), TerraCraft.OFF_X, TerraCraft.OFF_Y, null);
        //draw water/other foreground things
        for(int i = 0; i < blocksDrawAfter.size(); i++){
            g.drawImage(world.getBlockImage(blocksDrawAfter.get(i)),
                        blocksDrawAfterSpot.get(i)[0],
                        blocksDrawAfterSpot.get(i)[1],
                        null);
        }
        blocksDrawAfter.clear();
        blocksDrawAfterSpot.clear();
        //draw digging "animation"
        if(MouseLogic.isPressing() && TerraCraft.mouseHandler.getButton() == MouseHandler.LEFT){
            temp = MouseLogic.pressSize();
            g.setColor(Color.RED);
            g.setStroke(circleStroke);
            g.drawOval(TerraCraft.mouseHandler.getX() - temp / 2, TerraCraft.mouseHandler.getY() - temp / 2, temp, temp);
        }
        //draw text
        g.setColor(Color.BLACK);
        loc = new Location(TerraCraft.player.getX(), TerraCraft.player.getY());
        g.drawString("Chunk: " + world.getChunkAt(loc).getChunkNumber(), TerraCraft.SIZE_X - 80, 20);
        g.drawString("X: " + (int)(TerraCraft.player.getX()/20), 30, 20);
        g.drawString("Y: " + (int)(TerraCraft.player.getY()/20), 30, 40);
        //pixel point version
        //g.drawString("X: " + TerraCraft.player.getX(), 40, 20);
        //g.drawString("Y: " + TerraCraft.player.getY(), 40, 40);
        
    }

}
