package com.connormahaffey.TerraCraft;

/**
 *
 * @author Connor
 */
public class MouseLogic extends Logic implements Runnable{

    private final int xMin = 270, xMax = 450, yMin = 160, yMax = 320;
    private static double pressPercentage = 1;
    private static boolean pressing, instaBreak;
    private World world;

    public MouseLogic(World world){
        this.world = world;
    }

    public void run() {
        long time = System.currentTimeMillis(), oldTime = time, change = 0;
        long pressTime = 0;
        int button, currentButton = -1;
        int x, y;
        Location location;
        Block block, previousBlock = null;
        Chunk chunk;

        while(running){
            time = System.currentTimeMillis();
            change = time - oldTime;
            oldTime = time;
            if(pause){
                doPause();
                time = System.currentTimeMillis();
                change = time - oldTime;
                oldTime = time;
            }

            button = TerraCraft.mouseHandler.getButton();
            x = TerraCraft.mouseHandler.getX();
            y = TerraCraft.mouseHandler.getY();

            if(currentButton != button){
                currentButton = button;
                pressTime = 0;
                pressPercentage = 0;
            }

            if(TerraCraft.keyHandler.getMode() == KeyHandler.MODE_GAME && x >= xMin && x <= xMax && y >= yMin && y <= yMax){
                if(currentButton == MouseHandler.LEFT){
                    pressTime += change;
                    x = x - TerraCraft.OFF_X + (int)Math.floor(TerraCraft.player.getX());
                    y = - (TerraCraft.SIZE_Y - y - TerraCraft.HALF_Y - (int)Math.floor(TerraCraft.player.getY()));
                    location = new Location(x, y);
                    chunk = world.getChunkAt(location);
                    if(chunk != null){
                        block = chunk.getBlock(location);
                        if(previousBlock != null && block.getLocation().isEqualTo(previousBlock.getLocation())
                           && block.getLayer() == previousBlock.getLayer() && block.getHardness() != -1){
                            //remove dis later maaaaaaaeeeeeee
                            if(pressTime > block.getHardness() * 100 || instaBreak){
                                blockBreak(chunk, block);
                                if(block.getLayer() == Block.FOREGROUND || block.getType() == Block.WOOD){
                                   TerraCraft.player.addBlock(block.getType());
                                }
                                pressPercentage = 1;
                                pressing = false;
                            }
                            else{
                                pressPercentage = 1 - pressTime / (block.getHardness() * 100);
                                pressing = true;
                            }
                        }
                        else{
                            pressTime = change;
                        }
                        previousBlock = block;
                    }
                }
                else if(currentButton == MouseHandler.RIGHT){
                    x = x - TerraCraft.OFF_X + (int)Math.floor(TerraCraft.player.getX());
                    y = - (TerraCraft.SIZE_Y - y - TerraCraft.HALF_Y - (int)Math.floor(TerraCraft.player.getY()));
                    location = new Location(x, y);
                    chunk = world.getChunkAt(location);
                    block = chunk.getBlock(location);
                    if(block.getType() == Block.AIR || block.getLayer() == Block.BACKGROUND){
                        if(TerraCraft.player.hasBlock()){
                            chunk.replace(block, new Block(TerraCraft.player.getBlock(), Block.FOREGROUND, true, block.getLocation(), block.getType()));
                        }
                    }
                }
            }

            logicWait((int)(Logic.WAIT_PERIOD - (time - System.currentTimeMillis())));
        }

        finished = true;
    }
    /**
     * Changes blocks after you break them. For most, this will make it "air" - but
     * @param chunk chunk block resides in
     * @param block block to change
     */
    private void blockBreak(Chunk chunk, Block block){
        if(block.getLayer() == Block.FOREGROUND){
            if(block.isPlacedByPlayer()){
                if(block.getOldBackgroundType() == Block.AIR){
                    chunk.replace(block, new Block(Block.AIR, Block.FOREGROUND, true, block.getLocation()));
                }
                else{
                    chunk.replace(block, new Block(block.getOldBackgroundType(), Block.BACKGROUND, false, block.getLocation()));
                }
                
            }
            else{
                chunk.replace(block, new Block(block.getType(), Block.BACKGROUND, false, block.getLocation()));
            }
        }
        else{
            if(block.isPlacedByPlayer()){
                chunk.replace(block, new Block(Block.AIR, Block.BACKGROUND, false, block.getLocation()));
            }
        }
        
    }

    public static int pressSize(){
        if(pressPercentage == 0){
            return 0;
        }
        else{
            return (int) (pressPercentage * (World.BLOCK_SIZE * 2 / 3) + 2);
        }
    }

    public static boolean isPressing(){
        return pressing;
    }

    public static void toggleInstaBreak(){
        instaBreak = !instaBreak;
    }

    public static void setInstaBreak(boolean insta){
        instaBreak = insta;
    }

    public static boolean getInstaBreak(){
        return instaBreak;
    }

}
