package com.connormahaffey.TerraCraft;

import java.awt.Image;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

/**
 *
 * @author Connor
 */
public class World {

    public static final int CHUNK_WIDTH = 8, CHUNK_HEIGHT = 120, BLOCK_SIZE = 20,
                            CHUNK_WIDTH_PIXELS = CHUNK_WIDTH * BLOCK_SIZE, CHUNK_HEIGHT_PIXELS = CHUNK_HEIGHT * BLOCK_SIZE;

    private final int MOUNTAIN_CHANCE = 1200, CAVE_SURFACE_CHANCE = 1000;//75 out of 100,000

    private ArrayList<Chunk> chunks;
    private ArrayList<Chunk> unloadedChunks;
    private FileHandler fileHandler;
    private ArrayList<Image> blockImages, backgroundBlockImages;

    public World(){
        chunks = new ArrayList<Chunk>();
        unloadedChunks = new ArrayList<Chunk>();
        fileHandler = new FileHandler("bin" + FileHandler.separator);
        blockImages = new ArrayList<Image>();
        backgroundBlockImages = new ArrayList<Image>();
    }

    public void init(){
        BufferedImage blocks = fileHandler.getInternalImage("blocks.png");
        for(int i = 0; i < 5; i++){
            for(int j = 0; j < 5; j++){
                blockImages.add(fileHandler.getBlockImage(blocks, j * 40, i * 40));
            }
        }
        blocks = fileHandler.getInternalImage("backgroundblocks.png");
        for(int i = 0; i < 5; i++){
            for(int j = 0; j < 5; j++){
                backgroundBlockImages.add(fileHandler.getBlockImage(blocks, j * 40, i * 40));
            }
        }
    }

    public void load(){
        //make chunk you start in chunk you were last in
        if(fileHandler.exists("world.dat")){//change to ! for force generation
            String[] data = fileHandler.read("world.dat");
            double[] ps = fileHandler.getList(fileHandler.getValue(data, "PlayerPosition"));
            TerraCraft.player.setX(ps[0]);
            TerraCraft.player.setY(ps[1]);
            double[] pc = fileHandler.getList(fileHandler.getValue(data, "PlayerChunks"));
            for(int i = 0; i < pc.length; i++){
                if(fileHandler.exists("c" + ((int)pc[i]) + ".dat")){
                    loadChunkFromFile((int)pc[i]);
                }
                else{
                    generateChunk((int)pc[i]);
                }
            }
            double[] inv = fileHandler.getList(fileHandler.getValue(data, "Inventory"));
            TerraCraft.player.setInventory(inv);
        }
        else{
            //create default world.dat
            String[] data = {"PlayerPosition:0,-1300", "PlayerChunks:-3,-2,-1,0,1,2,3",
                             "CreativeMode:false", "InstantBreak:false", "SuperJump:false",
                             "Inventory:0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0"};
            fileHandler.write(data, "world.dat");
            //comment out -3,3 to see them being built
            generateChunk(0);
            generateChunk(-CHUNK_WIDTH_PIXELS);
            generateChunk(CHUNK_WIDTH_PIXELS);
            generateChunk(-2 * CHUNK_WIDTH_PIXELS);
            generateChunk(2 * CHUNK_WIDTH_PIXELS);
            generateChunk(-3 * CHUNK_WIDTH_PIXELS);
            generateChunk(3 * CHUNK_WIDTH_PIXELS);
        }
    }

    public void loadChunkFromFile(int num){
        Chunk chunk = new Chunk(num);
        Block block;
        String[] data = fileHandler.read("c" + num + ".dat");
        chunk.setDataFull(fileHandler.getList(data[0]));
        for(int i = 1; i < data.length; i++){
            block = new Block(fileHandler.getList(data[i]));
            chunk.addBlock(block);
        }
        chunks.add(chunk);
    }

    public void writeChunkToFile(Chunk chunk){
        int num = chunk.getChunkNumber();
        Block block;
        ArrayList<Block> blocks = chunk.getBlocks();
        String[] data = new String[blocks.size() + 1];
        data[0] = chunk.getDataToString();
        for(int i = 0; i < blocks.size(); i++){
            block = blocks.get(i);
            data[i + 1] = block.toString();
        }
        fileHandler.write(data, "c" + num + ".dat");
    }

    public Image getBlockImage(Block block){
        if(block.getLayer() == Block.FOREGROUND){
            return blockImages.get(block.getType() - 1);
        }
        else{
            return backgroundBlockImages.get(block.getType() - 1);
        }
    }

    public ArrayList<Chunk> getChunks(){
        return chunks;
    }

    public void addUnloadedChunk(Chunk chunk){
        unloadedChunks.add(chunk);
        chunks.remove(chunk);
    }

    public ArrayList<Chunk> getUnloadedChunks(){
        return unloadedChunks;
    }

    public boolean existsChunk(int position){
        position = position / CHUNK_WIDTH_PIXELS;
        if(fileHandler.exists("c" + position + ".dat")){
            return true;
        }
        else{
            return false;
        }
    }

    public Chunk getChunk(int position){
        for(int i = 0; i < chunks.size(); i++){
            if(chunks.get(i).getChunkNumber() == position){
                return chunks.get(i);
            }
        }

        return null;
    }

    public Chunk getChunkAt(Location loc){
        for(int i = 0; i < chunks.size(); i++){
            //spot could be changed by world generator, shifted one or left mid check
            //rare but add temp variable if that's the case, but not GLOBAL unless synchronized
            if(chunks.get(i).isWithinRange(loc.getXInt())){
                return chunks.get(i);
            }
        }

        return null;
    }

    public void generateChunk(int startingPoint){
        int chunkNum = (int)(startingPoint / CHUNK_WIDTH_PIXELS);
        Chunk chunk = new Chunk(chunkNum);
        Block block = null, block2 = null;
        Location loc;
        //put temp variables up here
        Chunk tempChunk, tempChunk2;
        int spot, min, max, change, start, end;
        //should always create all blocks in a chunk first, even if they are all air
        for(int i = 0; i < CHUNK_WIDTH; i++){
            for(int j = 0; j < CHUNK_HEIGHT; j++){
                loc = new Location(startingPoint + BLOCK_SIZE * i, -(BLOCK_SIZE * j + BLOCK_SIZE));
                if(j < 12){
                    block = new Block(Block.BEDROCK, Block.FOREGROUND, false, loc);
                }
                else if(j >= 12 && j < 44){
                    block = new Block(Block.STONE, Block.FOREGROUND, false, loc);
                }
                else if(j >= 44 && j < 119){
                    block = new Block(Block.AIR, Block.FOREGROUND, false, loc);
                }
                else{
                    block = new Block(Block.CLOUD, Block.FOREGROUND, false, loc);
                }
                chunk.addBlock(block);
            }
        }
        /*GENERATOR
        Taken data spots:
        0 = chunk right surface start spot
        1 = chunk left surface start spot
        2 = max value of surface change
        3 =
        4 = chunk right surface start spot
        5 = chunk left surface start spot
        6 = cave change value
        7 =
        8 =
        9 =
        problem - never goes back down because min is reset at end of chunk
        you probably want to look down there first
        getting the height of the last hill so there aren't huge dips every chunk - from chunk file*/
        tempChunk = getChunk(chunkNum - 1);//chunk to left
        tempChunk2 = getChunk(chunkNum + 1);//chunk to right
        //these ifs will affect the "order" chunks are changed, left to right or right to left
        if(tempChunk != null){
            spot = (int)tempChunk.getData()[1];
            start = 0;
            change = 1;
            end = CHUNK_WIDTH;
            max = (int)tempChunk.getData()[2];
        }
        else if(tempChunk2 != null){
            spot = (int)tempChunk2.getData()[0];
            start = CHUNK_WIDTH - 1;
            change = -1;
            end = -1;
            max = (int)tempChunk2.getData()[2];
        }
        else{
            //none
            spot = 45;//56
            start = 0;
            change = 1;
            end = CHUNK_WIDTH;
            max = 2;
        }
        min = -3;
        //actually change the blocks
        for(int i = start; i != end; i += change){//for the width of the chunk
            spot += getRandom(min, max);//get a random value and add it to the current spot
            for(int j = 44; j < spot - 10; j++){//above draws dirt to 48, draw value above that - stone section
                loc = new Location(startingPoint + BLOCK_SIZE * i, -(BLOCK_SIZE * (j) + BLOCK_SIZE));
                block = new Block(Block.STONE, Block.FOREGROUND, false, loc);
                chunk.replace(chunk.getBlock(loc), block);
            }
            for(int j = spot - 15; j < spot - 10; j++){//above draws dirt to 48, draw value above that - stone section
                loc = new Location(startingPoint + BLOCK_SIZE * i, -(BLOCK_SIZE * (j) + BLOCK_SIZE));
                if(getRandom(1,3) == 2){
                    block = new Block(Block.DIRT, Block.FOREGROUND, false, loc);
                }
                else{
                    block = new Block(Block.STONE, Block.FOREGROUND, false, loc);
                }
                chunk.replace(chunk.getBlock(loc), block);
            }
            for(int j = spot - 10; j < spot - 5; j++){//above draws dirt to 48, draw value above that - stone section
                loc = new Location(startingPoint + BLOCK_SIZE * i, -(BLOCK_SIZE * (j) + BLOCK_SIZE));
                if(getRandom(1,3) == 2){
                    block = new Block(Block.STONE, Block.FOREGROUND, false, loc);
                }
                else{
                    block = new Block(Block.DIRT, Block.FOREGROUND, false, loc);
                }
                chunk.replace(chunk.getBlock(loc), block);
            }
            for(int j = spot - 5; j < spot; j++){//dirt section
                loc = new Location(startingPoint + BLOCK_SIZE * i, -(BLOCK_SIZE * (j) + BLOCK_SIZE));
                if(getRandom(1,5)==3)
                    block = new Block(Block.SAND, Block.FOREGROUND, false, loc);
                else
                    block = new Block(Block.DIRT, Block.FOREGROUND, false, loc);
                chunk.replace(chunk.getBlock(loc), block);
            }
            //add grass on top of that
            loc = new Location(startingPoint + BLOCK_SIZE * i, -(BLOCK_SIZE * (spot) + BLOCK_SIZE));
            block = new Block(Block.GRASS, Block.FOREGROUND, false, loc);
            chunk.replace(chunk.getBlock(loc), block);
            //set the starting point on the left
            if(i == 0){
                if(chunk.getChunkNumber() == 0){
                    //set the player's starting point to this height if you are generating the center chunk (first run)
                    TerraCraft.player.setY(block.getLocation().getY() - World.BLOCK_SIZE);
                }
                //set left data spot
                chunk.setData(0, spot);
            }
            //make sure it is air on top of that
            for(int j = spot + 1; j < CHUNK_HEIGHT - 10; j++){
                loc = new Location(startingPoint + BLOCK_SIZE * i, -(BLOCK_SIZE * (j) + BLOCK_SIZE));
                block = new Block(Block.AIR, Block.FOREGROUND, false, loc);
                chunk.replace(chunk.getBlock(loc), block);
            }
            //set right data spot
            if(i == 7){
                chunk.setData(1, spot);
            }
            //if it starts to get too close to the top
            if(spot > CHUNK_HEIGHT - 40){
                if(max == 4){
                    //mountain top
                    max = 0;
                }
                else{
                    max = 1;
                }
            }
            //or too close to the bottom
            if(spot < 50){
                if(max == 0){
                    //mountain end
                    max = 2;
                }
                else{
                   max = 3;
                }
            }
            //spawn mountain
            if(getRandom(0, 100000) < MOUNTAIN_CHANCE && max != 4 && max != 0){
                max = 4;
                //mountain spawn
            }
        }

        chunk.setData(2, max);

        // <caves>-------------------------------------------
        if(tempChunk != null){
            spot = (int)tempChunk.getData()[5];
            start = 0;
            change = 1;
            end = CHUNK_WIDTH;
            max = (int)tempChunk.getData()[6];
        }
        else if(tempChunk2 != null){
            spot = (int)tempChunk2.getData()[4];
            start = CHUNK_WIDTH - 1;
            change = -1;
            end = -1;
            max = (int)tempChunk2.getData()[6];
        }
        else{
            //none
            spot = getRandom(20, 40);
            start = 0;
            change = 1;
            end = CHUNK_WIDTH;
            max = 0;
        }

        int amount = 3, lastAmount = 3;
        for(int i = start; i != end; i += change){
            amount = getRandom(2, 5);
            if(Math.abs(lastAmount - amount) > 2){
                if(amount > 4){
                    amount--;
                }
                else{
                    amount++;
                }
            }
            lastAmount = amount;
            spot += getRandom(-1 + max, 1 + max);
            for(int j = 0; j < amount; j++){
                
                loc = new Location(startingPoint + BLOCK_SIZE * i, -(BLOCK_SIZE * (spot + j) + BLOCK_SIZE));
                block = new Block(Block.STONE, Block.BACKGROUND, false, loc);
                block2 = chunk.getBlock(loc);
                chunk.replace(block2, block);

                if(block2.getType() == Block.AIR){
                    amount = 2;
                    if(getRandom(1, 10) < 4){
                        //decrease
                        max = -2;
                    }
                    else{
                        //new cave spot
                        max = 0;
                        spot = getRandom(20, 40);
                    }
                }
            }

            if(i == 0){
                chunk.setData(4, spot);
            }

            if(i == 7){
                chunk.setData(5, spot);
            }

            if(getRandom(0, 100000) < CAVE_SURFACE_CHANCE){ // usually 100000
                max = 1;
            }

            if(spot < 15 && max == -1){
                max = 0;
            }
            
        }

        if(max == -2){
            max = -1;
        }

        chunk.setData(6, max);

        // </caves>-------------------------------------------
        
        chunks.add(chunk);
    }

    private int getRandom(int min, int max){
        max = max - min + 1;
        return (int)(Math.random() * max + min);
    }

    public String[] getData(){
        return fileHandler.read("world.dat");
    }

    public void setData(String[] data){
        fileHandler.write(data, "world.dat");
    }
}
