package com.connormahaffey.TerraCraft;

import java.util.ArrayList;

/**
 *
 * @author Connor
 */
public class WorldLogic extends Logic implements Runnable{

    //this thread will run slower than all other logic threads because it doesn't need to be perfectly in-sync

    private final int WAIT = Logic.WAIT_PERIOD * 5;
    private Game game;
    private World world;

    public WorldLogic(Game game, World world){
        this.game = game;
        this.world = world;
    }

    public void run() {
        ArrayList<Chunk> chunks, unloadedChunks;
        Location location;
        Chunk chunk;
        int spot = 0, min = 0, max = 0, lastChunkSpot, temp;
        world.init();
        world.load();
        lastChunkSpot = world.getChunkAt(new Location(TerraCraft.player.getX(), TerraCraft.player.getY())).getChunkNumber();
        game.initGameScreen();
        long time = System.currentTimeMillis(), oldTime = time, change = 0;
        while(running){
            time = System.currentTimeMillis();
            change = time - oldTime;
            oldTime = time;
            if(pause){
                doPause();
                time = System.currentTimeMillis();
                oldTime = time;
                change = WAIT;
            }

            location = new Location(TerraCraft.player.getX(), TerraCraft.player.getY());
            spot = world.getChunkAt(location).getChunkNumber();
            if(spot != lastChunkSpot){
                lastChunkSpot = spot;
                chunks = world.getChunks();
                chunk = chunks.get(0);
                min = chunk.getChunkNumber();
                max = chunk.getChunkNumber();
                for(int i = 1; i < chunks.size(); i++){
                    chunk = chunks.get(i);
                    if(chunk.getChunkNumber() < min){
                        min = chunk.getChunkNumber();
                    }
                    if(chunk.getChunkNumber() > max){
                        max = chunk.getChunkNumber();
                    }
                }

                if(Math.abs(spot - min) > Math.abs(max - spot)){
                    temp = (max + 1) * World.CHUNK_WIDTH_PIXELS;
                    if(world.existsChunk(temp)){
                        world.loadChunkFromFile(temp / World.CHUNK_WIDTH_PIXELS);
                    }
                    else{
                        world.generateChunk(temp);
                    }
                    
                    chunk = world.getChunk(min);
                    world.addUnloadedChunk(chunk);
                }
                else{
                    temp = (min - 1) * World.CHUNK_WIDTH_PIXELS;
                    if(world.existsChunk(temp)){
                        world.loadChunkFromFile(temp / World.CHUNK_WIDTH_PIXELS);
                    }
                    else{
                        world.generateChunk(temp);
                    }
                    chunk = world.getChunk(max);
                    world.addUnloadedChunk(chunk);
                }
            }

            unloadedChunks = world.getUnloadedChunks();

            if(!unloadedChunks.isEmpty()){
                chunk = unloadedChunks.remove(0);
                world.writeChunkToFile(chunk);
            }

            logicWait((int)(WAIT - (time - System.currentTimeMillis())));
        }
        finished = true;
    }

    @Override
    public void stop(){
        running = false;
        ArrayList<Chunk> chunks = new ArrayList<Chunk>();
        chunks.addAll(world.getChunks());
        chunks.addAll(world.getUnloadedChunks());
        for(int i = 0; i < chunks.size(); i++){
            world.writeChunkToFile(chunks.get(i));
        }
        String[] data = world.getData();
        data[0] = "PlayerPosition:" + TerraCraft.player.getX() + "," + TerraCraft.player.getY();
        data[1] = "PlayerChunks:";
        for(int i = 0; i < world.getChunks().size(); i++){
            data[1] += world.getChunks().get(i).getChunkNumber() + ",";
        }
        data[2] = "CreativeMode:" + TerraCraft.player.isCreative();
        data[3] = "InstantBreak:" + MouseLogic.getInstaBreak();
        data[4] = "SuperJump:" + PlayerLogic.isSuperJump();
        data[5] = "Inventory:";
        for(int i = 0; i < TerraCraft.player.getInventory().length; i++){
            data[5] += TerraCraft.player.getInventory()[i] + ",";
        }
        world.setData(data);
    }
}
